#pragma once

#include "public.sdk/source/vst/vstaudioeffect.h"
#include "z_libpd.h"

typedef struct _pdparam
{
    float pdp_f;
    int pdp_dirty;
} t_pdparam;

namespace Steinberg {

class PdProcessor : public Steinberg::Vst::AudioEffect
{
public:
    PdProcessor ();
    ~PdProcessor () SMTG_OVERRIDE;

    /* Creator function */
    static Steinberg::FUnknown* createInstance (void* /*context*/) 
    { 
        return (Steinberg::Vst::IAudioProcessor*)new PdProcessor; 
    }

    /* AudioEffect overrides: */

        /* Called at first after constructor */
    Steinberg::tresult PLUGIN_API initialize
        (Steinberg::FUnknown* context) SMTG_OVERRIDE;
    
        /* Called at the end before destructor */
    Steinberg::tresult PLUGIN_API terminate () SMTG_OVERRIDE;
    
        /* Switch the Plug-in on/off */
    Steinberg::tresult PLUGIN_API setActive
        (Steinberg::TBool state) SMTG_OVERRIDE;

        /* called before any process call */
    Steinberg::tresult PLUGIN_API setupProcessing
        (Steinberg::Vst::ProcessSetup& newSetup) SMTG_OVERRIDE;

    Steinberg::tresult PLUGIN_API setBusArrangements
        (Vst::SpeakerArrangement* inputs, int32 numIns,
            Vst::SpeakerArrangement* outputs, int32 numOuts) SMTG_OVERRIDE;

        /* Asks if a given sample size is supported */
    Steinberg::tresult PLUGIN_API canProcessSampleSize
        (Steinberg::int32 symbolicSampleSize) SMTG_OVERRIDE;

        /* the process call */
    Steinberg::tresult PLUGIN_API process
        (Steinberg::Vst::ProcessData& data) SMTG_OVERRIDE;

        /* For persistence */
    Steinberg::tresult PLUGIN_API setState
        (Steinberg::IBStream* state) SMTG_OVERRIDE;
    Steinberg::tresult PLUGIN_API getState
        (Steinberg::IBStream* state) SMTG_OVERRIDE;

    Steinberg::tresult PLUGIN_API notify(Vst::IMessage *message) SMTG_OVERRIDE;

    void setfilename(const char *name, const char *dir);
    void defineparam(int argc, t_atom *argv);
    void quit( void);

    t_pdparam pd_paramvec[PDNPARAMS+1];
    int pd_ninbus = 0;
    int pd_inbusvec[PDMAXBUS] = {0};
    int pd_noutbus = 0;
    int pd_outbusvec[PDMAXBUS] = {0};
    Vst::ProcessData *datap = 0;

protected:

    t_pdinstance *pd1 = 0;
    int pd_vis = 0;
    int pd_open = 0;
    char pd_filename[MAXPDSTRING] = {0};
    char pd_filedir[MAXPDSTRING] = {0};
    void *pd_fromguihook;
    void *pd_frompdhook;
    double pd_lasttempo = -1;
    double pd_tickssincetempo = 1000;
    void *pd_dlopened;
};

} /* namespace Steinberg */

extern "C" {

static void pdmessage(const char *recv, const char *msg,
    int argc, t_atom *argv);
}
